//==========================================================================================
//
//		OpenNETCF.Win32.Notify
//		Copyright (c) 2003-2005, OpenNETCF.org
//
//		This library is free software; you can redistribute it and/or modify it under 
//		the terms of the OpenNETCF.org Shared Source License.
//
//		This library is distributed in the hope that it will be useful, but 
//		WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
//		FITNESS FOR A PARTICULAR PURPOSE. See the OpenNETCF.org Shared Source License 
//		for more details.
//
//		You should have received a copy of the OpenNETCF.org Shared Source License 
//		along with this library; if not, email licensing@opennetcf.org to request a copy.
//
//		If you wish to contact the OpenNETCF Advisory Board to discuss licensing, please 
//		email licensing@opennetcf.org.
//
//		For general enquiries, email enquiries@opennetcf.org or visit our website at:
//		http://www.opennetcf.org
//
//==========================================================================================
using System;
using System.ComponentModel;
using System.Runtime.InteropServices;

namespace OpenNETCF.Win32.Notify
{

	#region Notify

	/// <summary>
	/// Contains Notification related Methods
	/// </summary>
	public class Notify
	{
		private Notify()
		{
		}

		#region Run App At Event

		/// <summary>   
		/// This function starts running an application when a specified event occurs.   
		/// </summary>   
		/// <param name="appName">Name of the application to be started.</param>   
		/// <param name="whichEvent">Event at which the application is to be started.</param>   
		/// <seealso cref="T:OpenNETCF.Win32.Core.EventNotifications"/>   
		public static void RunAppAtEvent(string appName, NotificationEvent whichEvent)
		{
			if (!CeRunAppAtEvent(appName, (int) whichEvent))
			{
				throw new Exception("Cannot Set Notification Handler");
			}
		}

		#endregion

		#region Run App At Time

		/// <summary>   
		/// This function prompts the system to start running a specified application at a specified time.   
		/// </summary>   
		/// <param name="appName">Name of the application to be started.</param>   
		/// <param name="time">DateTime at which to run application.</param>
		/// <remarks>To cancel an existing RunAppATime request pass the application name and DateTime.MinValue</remarks>
		public static void RunAppAtTime(string appName, DateTime time)
		{
			byte[] timebytes;

			if (time == DateTime.MinValue)
			{
				//cancel request pass null
				timebytes = null;
			}
			else
			{
				//get native system time struct
				SystemTime st = SystemTime.FromDateTime(time);
				//get bytes
				timebytes = st.ToByteArray();
			}

			if (!CeRunAppAtTime(appName, timebytes))
			{
				throw new Exception("Cannot Set Notification Handler");
			}
		}

		#endregion

		#region Set User Notification

		/// <summary>
		/// Creates a new user notification.
		/// </summary>
		/// <param name="application">String that specifies the name of the application that owns this notification.</param>
		/// <param name="time">The time when the notification should occur.</param>
		/// <param name="notify">Notification object that describes the events that are to occur when the notification time is reached.</param>
		/// <returns>The handle to the notification indicates success.</returns>
		public static int SetUserNotification(string application, DateTime time, UserNotification notify)
		{
			return SetUserNotification(0, application, time, notify);
		}

		/// <summary>
		/// Edit an existing user notification.
		/// </summary>
		/// <param name="handle">Handle to the notification to overwrite.</param>
		/// <param name="application">String that specifies the name of the application that owns this notification.</param>
		/// <param name="time">The time when the notification should occur.</param>
		/// <param name="notify">Notification object that describes the events that are to occur when the notification time is reached.</param>
		/// <returns>The handle to the notification indicates success.</returns>
		public static int SetUserNotification(int handle, string application, DateTime time, UserNotification notify)
		{
			//call api function
			int outhandle =
				CeSetUserNotification(handle, application, SystemTime.FromDateTime(time).ToByteArray(), notify);

			//if invalid handle throw exception
			if (outhandle == 0)
			{
				throw new Win32Exception(Marshal.GetLastWin32Error(), "Error setting UserNotification");
			}
			return outhandle;
		}

		/// <summary>
		/// This function creates a new user notification.
		/// </summary>
		/// <param name="trigger">A UserNotificationTrigger that defines what event activates a notification.</param>
		/// <param name="notification">A UserNotification that defines how the system should respond when a notification occurs.</param>
		/// <returns>Handle to the notification event if successful.</returns>
		public static int SetUserNotification(UserNotificationTrigger trigger, UserNotification notification)
		{

			int outhandle = CeSetUserNotificationEx(0, trigger, null);

			//throw on invalid handle
			if (outhandle == 0)
			{
				throw new Win32Exception(Marshal.GetLastWin32Error(), "Error setting UserNotification");
			}
			return outhandle;
		}

		/// <summary>
		/// This function modifies an existing user notification.
		/// </summary>
		/// <param name="handle">Handle of the Notification to be modified</param>
		/// <param name="trigger">A UserNotificationTrigger that defines what event activates a notification.</param>
		/// <param name="notification">A UserNotification that defines how the system should respond when a notification occurs.</param>
		/// <returns>Handle to the notification event if successful.</returns>
		public static int SetUserNotification(int handle, UserNotificationTrigger trigger, UserNotification notification)
		{
			int outhandle = CeSetUserNotificationEx(handle, trigger, notification);

			//throw on invalid handle
			if (outhandle == 0)
			{
				throw new Win32Exception(Marshal.GetLastWin32Error(), "Error setting UserNotification");
			}
			return outhandle;
		}

		#endregion

		#region Clear User Notification

		/// <summary>
		/// Deletes a registered user notification that was created by a previous call to the SetUserNotification function.
		/// </summary>
		/// <param name="handle">Handle to the user notification to delete.</param>
		/// <returns>TRUE indicates success. FALSE indicates failure.</returns>
		/// <remarks>ClearNotification does not operate on notifications that have occurred.</remarks>
		public static bool ClearUserNotification(int handle)
		{
			return CeClearUserNotification(handle);
		}

		#endregion

		#region Get User Notification

		/// <summary>
		/// Retrieves notification information associated with a handle.
		/// </summary>
		/// <param name="handle">Handle to the user notification to retrieve.</param>
		/// <returns>The requested UserNotification.</returns>
		public static UserNotificationInfoHeader GetUserNotification(int handle)
		{
			//buffer size
			int size = 0;

			//first query for buffer size required
			CeGetUserNotification(handle, 0, ref size, IntPtr.Zero);

			//create a marshallable buffer
			IntPtr buffer = Marshal.AllocHGlobal(size);

			//call native getter
			if (!CeGetUserNotification(handle, (uint) size, ref size, buffer))
			{
				throw new Win32Exception(Marshal.GetLastWin32Error(), "Error getting UserNotification");
			}

			IntPtr p_trigger = (IntPtr) Marshal.ReadInt32(buffer, 8);
			IntPtr p_notification = (IntPtr) Marshal.ReadInt32(buffer, 12);


			UserNotification notification = new UserNotification();
			if (p_notification != IntPtr.Zero)
			{
				notification.Action = (NotificationAction) Marshal.ReadInt32(p_notification, 0);
				notification.Title = Marshal.PtrToStringUni((IntPtr)Marshal.ReadInt32(p_notification, 4));
				notification.Text = Marshal.PtrToStringUni((IntPtr)Marshal.ReadInt32(p_notification, 8));
				notification.Sound = Marshal.PtrToStringUni((IntPtr)Marshal.ReadInt32(p_notification, 12));
			}

			UserNotificationTrigger trigger = new UserNotificationTrigger();

			if (p_trigger != IntPtr.Zero)
			{
				trigger.Type = (NotificationType)Marshal.ReadInt32(p_trigger, 4);
				trigger.Event = (NotificationEvent)Marshal.ReadInt32(p_trigger, 8);

				trigger.Application = Marshal.PtrToStringUni((IntPtr)Marshal.ReadInt32(p_trigger, 12));
				trigger.Arguments = Marshal.PtrToStringUni((IntPtr)Marshal.ReadInt32(p_trigger, 16));

				SYSTEMTIME st = (SYSTEMTIME)Marshal.PtrToStructure((IntPtr)(p_trigger.ToInt32() + 20), typeof(SYSTEMTIME));
				trigger.StartTime = st.ToDateTime();

				st = (SYSTEMTIME)Marshal.PtrToStructure((IntPtr)(p_trigger.ToInt32() + 36), typeof(SYSTEMTIME));
				trigger.EndTime = st.ToDateTime();
			}

			UserNotificationInfoHeader header = new UserNotificationInfoHeader();

			header.Status = (NotificationStatus) Marshal.ReadInt32(buffer, 4);
			header.Handle = handle;

			header.UserNotification = notification;
			header.UserNotificationTrigger = trigger;

			//free native memory
			Marshal.FreeHGlobal(buffer);

			return header;
		}

		#endregion

		#region Get User Notification Handles

		/// <summary>
		/// Returns an array of currently stored notifications.
		/// </summary>
		/// <returns>Array of currently stored notifications.</returns>
		public static int[] GetUserNotificationHandles()
		{
			int size = 0;
			//get size required
			if (!CeGetUserNotificationHandles(null, 0, ref size))
			{
				throw new Exception("Error retrieving handles");
			}
			//create array to fill
			int[] handles = new int[size];
			//this time pass the buffer to be filled
			if (!CeGetUserNotificationHandles(handles, size, ref size))
			{
				throw new Win32Exception(Marshal.GetLastWin32Error(), "Error retrieving handles");
			}

			//return populated handles array
			return handles;
		}

		#endregion

		#region Get User Notification Preferences

		/// <summary>
		/// This function queries the user for notification settings by displaying a dialog box showing options that are valid for the current hardware platform.
		/// </summary>
		/// <param name="hWnd">Handle to the parent window for the notification settings dialog box.</param>
		/// <returns>A UserNotification structure containing the user's notification settings.</returns>
		public static UserNotification GetUserNotificationPreferences(IntPtr hWnd)
		{
			UserNotification template = new UserNotification();

			return GetUserNotificationPreferences(hWnd, template);
		}

		/// <summary>
		/// This function queries the user for notification settings by displaying a dialog box showing options that are valid for the current hardware platform.
		/// </summary>
		/// <param name="hWnd">Handle to the parent window for the notification settings dialog box.</param>
		/// <param name="template">UserNotification structure used to populate the default settings.</param>
		/// <returns>A UserNotification structure containing the user's notification settings.</returns>
		public static UserNotification GetUserNotificationPreferences(IntPtr hWnd, UserNotification template)
		{
			template.Sound = new string('\0', 255);

			if (!CeGetUserNotificationPreferences(hWnd, template))
			{
				throw new Win32Exception(Marshal.GetLastWin32Error(), "Could not get user preferences");
			}
			return template;
		}

		#endregion

		#region Handle App Notifications

		/// <summary>
		/// This function marks as "handled" all notifications previously registered by the given application that have occurred.
		/// </summary>
		/// <param name="application">The name of the application whose events are to be marked as "handled".
		/// This must be the name that was passed in to <see cref="M:OpenNETCF.Win32.Notify.Notify.SetUserNotification"/> as the owner of the notification.</param>
		public static void HandleAppNotifications(string application)
		{
			//throw exception on failure
			if (!CeHandleAppNotifications(application))
			{
				throw new Win32Exception(Marshal.GetLastWin32Error(), "Error clearing Application Notifications");
			}
		}

		#endregion

		#region Notify P/Invokes

		[DllImport("coredll.dll", EntryPoint="CeRunAppAtEvent", SetLastError=true)]
		private static extern bool CeRunAppAtEvent(string pwszAppName, int lWhichEvent);

		[DllImport("coredll.dll", EntryPoint="CeRunAppAtTime", SetLastError=true)]
		private static extern bool CeRunAppAtTime(string pwszAppName, byte[] lpTime);

		[DllImport("coredll.dll", EntryPoint="CeSetUserNotificationEx", SetLastError=true)]
		private static extern int CeSetUserNotificationEx(int hNotification, UserNotificationTrigger trigger, UserNotification userNotification);

		[DllImport("coredll.dll", EntryPoint="CeSetUserNotification", SetLastError=true)]
		private static extern int CeSetUserNotification(int hNotification, string pwszAppName, byte[] lpTime,
		                                                UserNotification userNotification);

		[DllImport("coredll.dll", EntryPoint="CeClearUserNotification", SetLastError=true)]
		private static extern bool CeClearUserNotification(int hNotification);

		[DllImport("coredll.dll", EntryPoint="CeGetUserNotification", SetLastError=true)]
		private static extern bool CeGetUserNotification(int hNotification, uint cBufferSize, ref int pcBytesNeeded,
		                                                 IntPtr pBuffer);

		[DllImport("coredll.dll", EntryPoint="CeGetUserNotificationHandles", SetLastError=true)]
		private static extern bool CeGetUserNotificationHandles(int[] rghNotifications, int cHandles, ref int pcHandlesNeeded);

		[DllImport("coredll.dll", EntryPoint="CeGetUserNotificationPreferences", SetLastError=true)]
		private static extern bool CeGetUserNotificationPreferences(IntPtr hWndParent, UserNotification notification);

		[DllImport("coredll.dll", EntryPoint="CeHandleAppNotifications", SetLastError=true)]
		private static extern bool CeHandleAppNotifications(string appName);

		#endregion
	}

	#endregion


	#region User Notification Info Header

	/// <summary>
	/// Contains information about notification events.
	/// </summary>
	public class UserNotificationInfoHeader
	{
		int _Handle;
		NotificationStatus status;
		private UserNotification notify;
		private UserNotificationTrigger trigger;

		/// <summary>
		/// Handle to the notification.
		/// </summary>
		public int Handle
		{
			get
			{
				return _Handle;
			}
			set
			{
				_Handle = value;
			}
		}

		/// <summary>
		/// Indicates current state of the notification.
		/// </summary>
		public NotificationStatus Status
		{
			get
			{
				return status;
			}
			set
			{
				status = value;
			}
		}

		/// <summary>
		/// The UserNotificationTrigger object
		/// </summary>
		public UserNotificationTrigger UserNotificationTrigger
		{
			get
			{
				return trigger;
			}
			set
			{
				trigger = value;
			}
		}

		/// <summary>
		/// The UserNotification object.
		/// </summary>
		public UserNotification UserNotification
		{
			get
			{
				return notify;
			}
			set
			{
				notify = value;
			}
		}

	}

	#endregion


	#region User Notification

	/// <summary>
	/// This structure contains information used to initialize the user notifications settings dialog box, and receives the users notification preferences entered by way of the dialog box.
	/// Also used when setting a user notification. 
	/// </summary>
	/// 
	[StructLayout(LayoutKind.Sequential)]
	public class UserNotification 
	{
		int _Action;

		[MarshalAs(UnmanagedType.LPWStr)]
		private string _Title;

		[MarshalAs(UnmanagedType.LPWStr)]
		private string _Text;

		[MarshalAs(UnmanagedType.LPWStr)]
		private string _Sound;

		private int _MaxSound;
		int _Reserved;

		/// <summary>
		/// Any combination of the <see cref="T:OpenNETCF.Win32.Notify.NotificationAction"/> members.  
		/// </summary>
		/// <value>Flags which specifies the action(s) to be taken when the notification is triggered.</value>
		/// <remarks>Flags not valid on a given hardware platform will be ignored.</remarks>
		public NotificationAction Action
		{
			get { return (NotificationAction)_Action; }
			set { _Action = (int)value; }
		}

		/// <summary>
		/// Required if NotificationAction.Dialog is set, ignored otherwise
		/// </summary>
		public string Title
		{
			get
			{
				return _Title;
			}
			set
			{
				_Title = value;
			}
		}

		/// <summary>
		/// Required if NotificationAction.Dialog is set, ignored otherwise.
		/// </summary>
		public string Text
		{
			get
			{
				return _Text;
			}
			set
			{
				_Text = value;
			}
		}

		/// <summary>
		/// Sound string as supplied to PlaySound.
		/// </summary>
		/// <remarks>SetUserNotification() ignores it if the <see cref="P:OpenNETCF.Win32.Notify.UserNotification.Action">Action property</see> does not contain <see cref="T:OpenNETCF.Win32.Notify.NotifyAction">NotifyAction.Sound</see>.</remarks>
		public string Sound
		{
			get
			{
				return _Sound;
			}
			set
			{
				_Sound = value;
			}
		}

		/// <summary>
		/// Maximum length of sound parameter.
		/// </summary>
		public int MaxSound
		{
			get { return _MaxSound; }
			set { _MaxSound = value; }
		}
	}

	#endregion


	#region User Notification Trigger

	/// <summary>
	/// Defines what event activates a notification.
	/// </summary>
	[StructLayout(LayoutKind.Sequential)]
	public class UserNotificationTrigger
	{
		internal int _Size = 52;
		int _Type;
		int _Event;

		[MarshalAs(UnmanagedType.LPWStr)]
		string _Application = string.Empty;

		[MarshalAs(UnmanagedType.LPWStr)]
		string _Arguments;

		internal SYSTEMTIME _StartTime = new SYSTEMTIME();
		internal SYSTEMTIME _EndTime = new SYSTEMTIME();

		/// <summary>
		/// Specifies the type of notification.
		/// </summary>
		public NotificationType Type
		{
			get { return (NotificationType)_Type; }
			set { _Type = (int)value; }
		}

		/// <summary>
		/// Specifies the type of event should Type = Event.
		/// </summary>
		public NotificationEvent Event
		{
			get
			{
				return (NotificationEvent)_Event;
			}
			set
			{
				_Event = (int)value;
			}
		}

		/// <summary>
		/// Name of the application to execute.
		/// </summary>
		public string Application
		{
			get
			{
				return _Application;
			}
			set
			{
				_Application = value;
			}
		}

		/// <summary>
		/// Command line (without the application name). 
		/// </summary>
		public string Arguments
		{
			get
			{
				return _Arguments;
			}
			set
			{
				_Arguments = value;
			}
		}

		/// <summary>
		/// Specifies the beginning of the notification period.
		/// </summary>
		public DateTime StartTime
		{
			get
			{
				return _StartTime.ToDateTime();
			}
			set
			{
				_StartTime.FromDateTime(value);
			}
		}

		public DateTime EndTime
		{
			get { return _EndTime.ToDateTime(); }
			set { _EndTime.FromDateTime(value); }
		}
	}

	#endregion

	#region Notification Enums

	#region Notification Event

	/// <summary>   
	/// System Event Flags   
	/// </summary>   
	public enum NotificationEvent : int
	{
		/// <summary>   
		/// No eventsremove all event registrations for this application.   
		/// </summary>   
		None = 0x00,
		/// <summary>   
		/// When the system time is changed.   
		/// </summary>   
		TimeChange = 0x01,
		/// <summary>   
		/// When data synchronization finishes.   
		/// </summary>   
		SyncEnd = 0x02,
		/// <summary>
		/// The user turned the AC power on.
		/// </summary>
		OnACPower = 0x03,
		/// <summary>
		/// The user turned the alternating current (AC) power off.
		/// </summary>
		OffACPower = 0x04,
		/// <summary>
		/// The device connected to a network.
		/// </summary>
		NetConnect = 0x05,
		/// <summary>
		/// The device disconnected from a network.
		/// </summary>
		NetDisconnect = 0x06,
		/// <summary>   
		/// When a PC Card device is changed.   
		/// </summary>   
		DeviceChange = 0x07,
		/// <summary>
		/// The device discovered a server by using infrared communications.
		/// </summary>
		IRDiscovered = 0x08,
		/// <summary>   
		/// When an RS232 connection is made.   
		/// </summary>   
		RS232Detected = 0x09,
		/// <summary>   
		/// When a full device data restore completes.   
		/// </summary>   
		RestoreEnd = 0x0A,
		/// <summary>   
		/// When the device wakes up.   
		/// </summary>   
		Wakeup = 0x0B,
		/// <summary>   
		/// When the time zone is changed.   
		/// </summary>   
		TimeZoneChange = 0x0C,
		/// <summary>
		/// When the machines name changes.
		/// Requires Windows CE.NET 4.2.
		/// </summary>
		MachineNameChange = 0x0D,
		/// <summary>
		/// RNDISFN interface is instantiated.
		/// Requires Windows CE 5.0.
		/// </summary>
		RndisFNDetected = 0x0E,
		/// <summary>
		/// The Internet Proxy used by the device has changed.
		/// Requires Windows CE 5.0.
		/// </summary>
		InternetProxyChange = 0x0f,
	}

	#endregion

	#region Notification Action

	/// <summary>
	/// Specifies the action to take when a notification event occurs.
	/// </summary>
	[Flags()]
	public enum NotificationAction : int
	{
		/// <summary>
		/// Flashes the LED.
		/// </summary>
		Led = 1,
		/// <summary>
		/// Vibrates the device.
		/// </summary>
		Vibrate = 2,
		/// <summary>
		/// Displays the user notification dialog box.
		/// </summary>
		Dialog = 4,
		/// <summary>
		/// Plays the sound specified.
		/// </summary>
		Sound = 8,
		/// <summary>
		/// Repeats the sound for 1015 seconds.
		/// </summary>
		Repeat = 16,
		/// <summary>
		/// Dialog box z-order flag.
		/// Set if the notification dialog box should come up behind the password.
		/// </summary>
		Private = 32,
	}

	#endregion

	#region Notification Status

	/// <summary>
	/// The status of the notification.
	/// </summary>
	public enum NotificationStatus : int
	{
		/// <summary>
		/// The notification is not currently active.
		/// </summary>
		Inactive = 0,
		/// <summary>
		/// The notification is currently active.
		/// </summary>
		Signalled = 1,
	}

	#endregion

	#region Notification Type

	/// <summary>
	/// Specifies the type of notification.
	/// </summary>
	public enum NotificationType :int
	{
		/// <summary>
		/// System event notification.
		/// </summary>
		Event = 1,
		/// <summary>
		/// Time-based notification.
		/// </summary>
		Time = 2,
		/// <summary>
		/// Time-based notification that is active for the time period between <see cref="M:OpenNETCF.Win32.Notify.NotificationTrigger.StartTime"/> and <see cref="M:OpenNETCF.Win32.Notify.NotificationTrigger.EndTime"/>.
		/// </summary>
		Period = 3,
		/// <summary>
		/// Equivalent to using the SetUserNotification function.
		/// The standard command line is supplied.
		/// </summary>
		ClassicTime = 4,
	}

	#endregion

	#region Notification Command Line

	/// <summary>
	/// Strings passed on the command line when an event occurs that the app has requested via CeRunAppAtEvent.  
	/// </summary>
	/// <remarks>Note that some of these strings will be used as the command line *prefix*, since the rest of the command line will be used as a parameter.</remarks>
	public enum NotificationCommandLine
	{
		/// <summary>
		/// String passed on the command line when an app is run as the result of a call to <see cref="M:OpenNETCF.Win32.Notify.Notify.RunAppAtTime"/>.
		/// </summary>
		AppRunAtTime,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application that "owns" a notification.  It is followed by a space, and the stringized version of the notification handle.
		/// </summary>
		AppRunToHandleNotification,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when the system time settings are changed.
		/// </summary>
		AppRunAfterTimeChange,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application after synchronisation.
		/// </summary>
		AppRunAfterSync,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when the device is connected to AC power.
		/// </summary>
		AppRunAtAcPowerOn,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when the AC power is disconnected.
		/// </summary>
		AppRunAtAcPowerOff,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when the device connects to a LAN.
		/// </summary>
		AppRunAtNetConnect,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when the device disconnects from a LAN.
		/// </summary>
		AppRunAtNetDisconnect,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application that "owns" a notification.  It is followed by a space, and the stringized version of the notification handle.
		/// </summary>
		AppRunAtDeviceChange,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when another device is discovered using IR.
		/// </summary>
		AppRunAtIrDiscovery,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when a serial port connection is attempted.
		/// </summary>
		AppRunAtRs232Detect,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application after a system restore.
		/// </summary>
		AppRunAfterRestore,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when the device wakes up from standby.
		/// </summary>
		AppRunAfterWakeup,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application when the device time-zone is changed.
		/// </summary>
		AppRunAfterTzChange,
		/// <summary>
		/// Prefix of the command line when the user requests to run the application after an extended event.
		/// </summary>
		AppRunAfterExtendedEvent,
	}

	#endregion

	#endregion
}